/*  This program consists of a base window coloured yellow.  When the mouse
 *  pointer is over this window and a mouse button is pressed, the coordinates
 *  of the pointer relative to the window is printed on the console window and a
 *  red window containing a green window is drawn at that point.  If tbe mouse
 *  button pressed is the left-hand mouse button, then the beep of the computer
 *  is also sounded.  If the right-hand mouse button is clicked over the green
 *  window, the text 'ouch!' is also printed on the display console window.
 *
 *  Coded by:  Ross Maloney
 *  Date:      June 2008
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>

#define arrow_width 16
#define arrow_height 16
static unsigned char arrow_bits[] = {
   0x00, 0x00, 0x06, 0x00, 0x1e, 0x00, 0x7c, 0x00, 0xfc, 0x01, 0xfc, 0x07,
   0xf8, 0x1f, 0xf8, 0x7f, 0xf8, 0x7f, 0xf0, 0x03, 0xf0, 0x07, 0xe0, 0x06,
   0xe0, 0x0c, 0xc0, 0x18, 0x80, 0x30, 0x00, 0x00};

#define mask_width 16
#define mask_height 16
static unsigned char mask_bits[] = {
   0x07, 0x00, 0x1f, 0x00, 0x7f, 0x00, 0xfe, 0x01, 0xfe, 0x07, 0xfe, 0x1f,
   0xfc, 0x3f, 0xfc, 0xff, 0xf8, 0xff, 0xf8, 0xff, 0xf8, 0x0f, 0xf0, 0x1f,
   0xf0, 0x3f, 0xe0, 0x7d, 0xe0, 0x79, 0xc0, 0x71};

int main(int argc, char *argv)
{
  Display        *mydisplay;
  XSetWindowAttributes  baseat, redat, greenat;
  Window         baseW, redW, greenW;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         abc, myevent;
  XColor         exact, closest;
  GC             baseGC;
  XGCValues      myGCValues;
  GC             Pgc;
  XGCValues      PGCValues;
  Pixmap         Ppattern, mask;
  unsigned long  Pmask;
  int            x, y;
  char *window_name = "Events";
  char *icon_name   = "Ev";
  int            screen_num, done;
  unsigned long  valuemask, red, green;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

	       /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "yellow", &exact, &closest);
  baseat.background_pixel = closest.pixel;
  baseat.border_pixel = BlackPixel(mydisplay, screen_num);
  baseat.event_mask = ButtonPressMask;
  valuemask = CWBackPixel | CWBorderPixel | CWEventMask;
  baseW = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                        300, 300, 350, 400, 3,
			DefaultDepth(mydisplay, screen_num), InputOutput,
			DefaultVisual(mydisplay, screen_num),
			valuemask, &baseat);

	       /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseW, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseW, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseW, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, baseW, &iconName);

	       /* 4.  establish window resources */
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "red", &exact, &closest);
  red = closest.pixel;
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "green", &exact, &closest);
  green = closest.pixel;
  Ppattern = XCreatePixmapFromBitmapData(mydisplay, baseW,
                               arrow_bits, arrow_width, arrow_height,
			       WhitePixel(mydisplay, screen_num),
			       BlackPixel(mydisplay, screen_num),
			       DefaultDepth(mydisplay, screen_num));
  mask = XCreatePixmapFromBitmapData(mydisplay, baseW,
                               mask_bits, mask_width, mask_height, 1, 0, 1);
  Pmask = GCForeground | GCBackground | GCClipMask;
  PGCValues.background = WhitePixel(mydisplay, screen_num);
  PGCValues.foreground = BlackPixel(mydisplay, screen_num);
  PGCValues.clip_mask = mask;
//  Pgc = XCreateGC(mydisplay, baseW, Pmask, &PGCValues);

	       /* 5.  create all the other windows needed */
//  valuemaks = CWBackPixel | CWBorderPixel | CWEventMask;
//  redat.background_pixel = closest;
//  redat.border_pixel = BlackPixel(mydisplay, screen_num);
//  valuemask = CWBackPixel | CWBorderPixel | CWEventMask;

	       /* 6.  select events for each window */
	       /* 7.  map the windows */
  XMapWindow(mydisplay, baseW);

	       /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &abc);
    switch(abc.type)  {
    case ButtonPress:
      if (abc.xbutton.button == Button1) XBell(mydisplay, 100);
      if (abc.xbutton.button == Button3 && abc.xbutton.window == greenW)
        printf("ouch!\n");
      x = abc.xbutton.x;
      y = abc.xbutton.y;
      if (abc.xbutton.window == baseW)  printf("Yellow window: ");
      if (abc.xbutton.window == redW)  printf("Red window: ");
      if (abc.xbutton.window == greenW)  printf("Green window: ");
      printf("x = %d  y = %d\n", x, y);
      redW = XCreateSimpleWindow(mydisplay, baseW, x, y, 100, 50, 1,
                             BlackPixel(mydisplay, screen_num), red);
      XMapWindow(mydisplay, redW);
      XSelectInput(mydisplay, redW, ButtonPressMask);
      greenW = XCreateSimpleWindow(mydisplay, redW, 10, 20, 40, 20, 1,
                              BlackPixel(mydisplay, screen_num), green);
      XMapWindow(mydisplay, greenW);
      XSelectInput(mydisplay, greenW, ButtonPressMask);
      if ( abc.xbutton.button == Button3 )  {
        x = abc.xbutton.x;
        y = abc.xbutton.y;
  Pgc = XCreateGC(mydisplay, redW, Pmask, &PGCValues);
	XSetClipOrigin(mydisplay, Pgc, x, y);
        XCopyPlane(mydisplay, Ppattern, redW, Pgc, 0, 0,
                   arrow_width, arrow_height, x, y, 1);
      }
      break;
    }
  }

	       /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, baseW);
  XDestroyWindow(mydisplay, baseW);
  XCloseDisplay(mydisplay);
}
