/*  This program implements hierarchical menus.  The base 400x400 pixel window
 *  contains two menu-bar buttons.  The button on the left hand side is
 *  connected to a menu of three menu items.  The bottom item of that menu is
 *  connected to a menu of two items, and the top one of those menu items is
 *  connected to another three item menu.  Each menu item is a blank window
 *  which changes colour when the mouse pointer moves over it.
 *
 *  As the mouse pointer enters a menu item window, it is highlighted and if a
 *  menu leads from it, that is displayed.  When the mouse pointer leaves a
 *  menu item, it ceases to be highlighted and any menu of menu items leading
 *  from it are removed from the display.  The left-hand mouse button is used to
 *  select a menu item.
 *
 *  Coded by:  Ross Maloney
 *  Date:      Jun 2009
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <stdio.h>

/* XPM */
static char *pointer[] = {
/* columns rows colors chars-per-pixel */
"16 16 2 1",
". c None",
"w c white",
"b c black",
/* pixels */
".ww.............",
".wbw............",
".wbbw...........",
".wbbbw..........",
".wbbbbw.........",
".wbbbbbw........",
".wbbbbbbw.......",
".wbbbbbbbw......",
".wbbbbbbbbw.....",
".wbbbbbbbbbw....",
".wbbwbbbwwww....",
".wbw.wbbw.......",
".ww..wbbw.......",
"......wbbw......",
"......wbbw......",
".......ww......."
};

int main(int argc, char *argv)
{
  Display        *mydisplay;
  Window         baseW;
  struct {
    Window  id;
    int     homemenu;
    int     shown;
    int     action;
  } W[10] = {
        {0, 1, 0, 2},
	{1, 1, 0, 1000},
	{2, 2, 0, 1000},
	{3, 2, 0, 1000},
	{4, 2, 0, 3},
	{5, 3, 0, 4},
	{6, 3, 0, 1000},
	{7, 4, 0, 1000},
	{8, 4, 0, 1100},
	{9, 4, 0, 1000}
    };
//  Window         W[10];
  XSetWindowAttributes  myat;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         baseEvent;
  GC             mygc;
  Pixmap         pattern, clipper;
  XpmAttributes  pointAt;
  char *window_name = "Hierarchy";
  char *icon_name   = "Hie";
  int            screen_num, done, status, i, window;
  unsigned long  mymask;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

	       /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  myat.border_pixel = 0xFF0000;    /* red */
  myat.background_pixel = 0xFFDEAD; /* navajo white */
//  myat.event_mask = ButtonPressMask | ExposureMask;
  myat.event_mask = ExposureMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  baseW = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                        350, 400, 400, 400, 2,
			DefaultDepth(mydisplay, screen_num), InputOutput,
			DefaultVisual(mydisplay, screen_num),
			mymask, &myat);

	       /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseW, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseW, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseW, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, baseW, &iconName);

	       /* 4.  establish window resources */
  pointAt.color_key = XPM_COLOR;
  pointAt.valuemask = XpmColorKey | XpmColorTable;
  done = XpmCreatePixmapFromData(mydisplay, baseW,
                     pointer, &pattern, &clipper, &pointAt);
  mygc = XCreateGC(mydisplay, baseW, 0, NULL);
  XSetForeground(mydisplay, mygc, WhitePixel(mydisplay, screen_num));
  XSetBackground(mydisplay, mygc, BlackPixel(mydisplay, screen_num));
  XSetClipMask(mydisplay, mygc, clipper);
  XSetClipOrigin(mydisplay, mygc, 0, 0);
	       /* 5.  create all the other windows needed */
  myat.background_pixel = 0xFFFFFF; /* white */
  myat.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask
                   | EnterWindowMask | LeaveWindowMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  W[0].id = XCreateWindow(mydisplay, baseW,
                          50, 50, 90, 20, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[1].id = XCreateWindow(mydisplay, baseW,
                          250, 100, 70, 30, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[2].id = XCreateWindow(mydisplay, baseW,
                          70, 60, 90, 20, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[3].id = XCreateWindow(mydisplay, baseW,
                          70, 80, 90, 20, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[4].id = XCreateWindow(mydisplay, baseW,
                          70, 100, 90, 20, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[5].id = XCreateWindow(mydisplay, baseW,
                          140, 110, 60, 10, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[6].id = XCreateWindow(mydisplay, baseW,
                          140, 120, 60, 10, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[7].id = XCreateWindow(mydisplay, baseW,
                          200, 110, 100, 30, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[8].id = XCreateWindow(mydisplay, baseW,
                          200, 140, 100, 30, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);
  W[9].id = XCreateWindow(mydisplay, baseW,
                          200, 170, 100, 30, 2,
			  DefaultDepth(mydisplay, screen_num), InputOutput,
			  DefaultVisual(mydisplay, screen_num),
			  mymask, &myat);

	       /* 6.  select events for each window */
	       /* 7.  map the windows */
  XMapWindow(mydisplay, baseW);
  for (i=0; i<10; i++)
    XMapWindow(mydisplay, W[i].id);

	       /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &baseEvent);
    for (i=0; i<10; i++)
      if ( W[i].id == baseEvent.xany.window )  {
        window = i;
	break;
      }
    switch (baseEvent.type)  {
    case  Expose:
      break;
    case  ButtonPress:
      printf("Button pressed in window %d\n", window);
//      XUngrabPointer(mydisplay, CurrentTime);
      XCopyArea(mydisplay, pattern, baseW, mygc, 0, 0, 16, 16, 100, 100);
      break;
    case  ButtonRelease:
      printf("Button released in window %d\n", window);
      XUnmapWindow(mydisplay, W[i].id);
      break;
    case  EnterNotify:
      printf("Window  %d entered\n", window);
      XSetWindowBackground(mydisplay, W[window].id, 0xFF0000);
      XClearWindow(mydisplay, W[window].id);
      XFlush(mydisplay);
      break;
    case  LeaveNotify:
      printf("Leaving window %d\n", window);
//      XSetWindowBackground(mydisplay, W[window].id, 0xFFFFFF);
//      XClearWindow(mydisplay, W[window].id);
//      XFlush(mydisplay);
      break;
    }
  }

	       /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, baseW);
  XDestroyWindow(mydisplay, baseW);
  XCloseDisplay(mydisplay);
}
