/* This program creates a button, labelled 'quit' located in a window.  Clicking
 * the mouse on this button terminates the execution of this program.  The
 * button has a red background and the labelling is in a yellow font.  The
 * window itself has a default white background.
 *
 * This program was the first to be written using the '9 Xlib programming steps'
 * indicated in the xlib.tex presentation.
 *
 * Coded by:  Ross Maloney
 * Date:      21 June 2006
 */

#include  <X11/Xlib.h>
#include  <X11/Xutil.h>
#include  <stdio.h>
#include  <string.h>

int main(int argc, char *argv[])
{
  Display        *mydisplay;
  XSetWindowAttributes  myat;
  Window         a, b, c ,d, e;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         myevent;
  GC             mygc;
  XGCValues      myvalues;
  char *window_name = "Relationships";
  char *icon_name   = "Rel";
  int            screen_num, done;
  unsigned long  valuemask;


                 /* 1. open connection to the server */
  mydisplay = XOpenDisplay("");
                 /* 2. create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  myat.background_pixel = WhitePixel(mydisplay, screen_num);
  myat.border_pixel = BlackPixel(mydisplay, screen_num);
  valuemask = CWBackPixel | CWBorderPixel | CWEventMask | CWDontPropagate;
  myat.event_mask = ButtonPressMask | ExposureMask | KeyPressMask;
//  myat.do_not_propagate_mask = ButtonPressMask;
  a = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                           200, 200, 350, 250, 2,
			   DefaultDepth(mydisplay, screen_num), InputOutput,
			   DefaultVisual(mydisplay, screen_num),
			   valuemask, &myat);
                 /* 3. inform the Window Manager with hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, a, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, a, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, a, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, a, &iconName);
                 /* 4. establish window resources */
                 /* 5. create any other windows needed */
  myat.border_pixel = BlackPixel(mydisplay, screen_num);
  myat.background_pixel = 0xffffff;
  myat.event_mask = ButtonPressMask | ExposureMask | KeyPressMask;
  b = XCreateWindow(mydisplay, a,
                         10, 10, 70, 20, 2,
			 DefaultDepth(mydisplay, screen_num), InputOutput,
			 DefaultVisual(mydisplay, screen_num),
			 valuemask, &myat);
//  myat.event_mask = ButtonPressMask | ExposureMask;
//  myat.event_mask = ExposureMask;
  myat.event_mask = ExposureMask | KeyPressMask;
//  myat.do_not_propagate_mask = ButtonPressMask;
  c = XCreateWindow(mydisplay, a,
                         10, 40, 325, 200, 2,
			 DefaultDepth(mydisplay, screen_num), InputOutput,
			 DefaultVisual(mydisplay, screen_num),
			 valuemask, &myat);
//  myat.event_mask = ButtonPressMask | ExposureMask | KeyPressMask;
  myat.event_mask = ExposureMask | KeyPressMask;
//  myat.event_mask = NoEventMask;
//  myat.do_not_propagate_mask = ButtonPressMask;
  d = XCreateWindow(mydisplay, c,
                         20, 20, 60, 20, 2,
			 DefaultDepth(mydisplay, screen_num), InputOutput,
			 DefaultVisual(mydisplay, screen_num),
			 valuemask, &myat);
  myat.event_mask = ButtonPressMask | ExposureMask | KeyPressMask;
  e = XCreateWindow(mydisplay, c,
                         120, 20, 60, 20, 2,
			 DefaultDepth(mydisplay, screen_num), InputOutput,
			 DefaultVisual(mydisplay, screen_num),
			 valuemask, &myat);
                 /* 6. select the desired events for all windows */
                 /* 7. map the windows */
  XMapWindow(mydisplay, a);
  XMapWindow(mydisplay, b);
  XMapWindow(mydisplay, c);
  XMapWindow(mydisplay, d);
  XMapWindow(mydisplay, e);
                 /* 8. enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &myevent);
    switch (myevent.type) {
    case Expose:
      break;
    case ButtonPress:
      if ( myevent.xany.window == a )  printf("window a\n");
      if ( myevent.xany.window == b )  printf("window b\n");
      if ( myevent.xany.window == c )  printf("window c\n");
      if ( myevent.xany.window == d )  printf("window d\n");
      if ( myevent.xany.window == e )  printf("window e\n");
      done = 1;
      break;
    }
  }
		 /* 9. clean up before exiting */
  XUnmapWindow(mydisplay, a);
  XDestroyWindow(mydisplay, a);
  XCloseDisplay(mydisplay);
}
