/*  This program does the equivalent of the Postscript example on page 2-16 of
 *  Smith(1990) which is:
 *     /Times-BoldItalic findfont
 *     72 scalefont
 *     setfont
 *     72 648 moveto
 *     (Typeface) show
 *     showpage
 *  except this program displays the same string twice.
 *
 *  Coded by:  Ross Maloney
 *  Date:      August 2006
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <string.h>

char  message[] = "Typeface";
char  fontName75[] = "-adobe-times-bold-i-normal--*-720-75-75-p-0-iso8859-1";
char  fontName100[] = "-adobe-times-bold-i-normal--*-720-100-100-p-0-iso8859-1";

main()
{
  Display        *mydisplay;
  XSetWindowAttributes  myat;
  Window         mywindow;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         myevent;
  XColor         exact, closest;
  GC             myGC75, myGC100;
  XGCValues      myGCvalues;
  XFontStruct    *fontDetail;
  char *window_name = "Fonter";
  char *icon_name   = "Font";
  int            screen_num, done;
  unsigned long  valuemask;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

               /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  myat.background_pixel = WhitePixel(mydisplay, screen_num);
  myat.border_pixel = BlackPixel(mydisplay, screen_num);
  myat.event_mask = ExposureMask;
  valuemask = CWBackPixel | CWBorderPixel | CWEventMask;
  mywindow = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                           60, 70, 500, 300, 3,
			   DefaultDepth(mydisplay, screen_num), InputOutput,
			   DefaultVisual(mydisplay, screen_num),
			   valuemask, &myat);

               /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, mywindow, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, mywindow, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, mywindow, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, mywindow, &iconName);
               /* 4.  establish window resources */
  if ( ( fontDetail = XLoadQueryFont(mydisplay, fontName75) ) == NULL )  {
    printf("Could not load font %s\n", fontName75);
    exit(1);
  }
  myGCvalues.foreground = BlackPixel(mydisplay, screen_num);
  myGCvalues.font = fontDetail->fid;
  valuemask = GCForeground | GCFont;
  myGC75 = XCreateGC(mydisplay, mywindow, valuemask, &myGCvalues);
  if ( ( fontDetail = XLoadQueryFont(mydisplay, fontName100) ) == NULL )  {
    printf("Could not load font %s\n", fontName100);
    exit(1);
  }
  myGCvalues.font = fontDetail->fid;
  myGC100 = XCreateGC(mydisplay, mywindow, valuemask, &myGCvalues);
               /* 5.  create all the other windows needed */
               /* 6.  select events for each window */
               /* 7.  map the windows */
  XMapWindow(mydisplay, mywindow);
               /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &myevent);
    switch (myevent.type)  {
    case  Expose:
      XDrawString(mydisplay, mywindow, myGC75, 10, 100, message,
                  strlen(message));
      XDrawString(mydisplay, mywindow, myGC100, 10, 200, message,
                  strlen(message));
      break;
    }
  }
               /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, mywindow);
  XDestroyWindow(mydisplay, mywindow);
  XCloseDisplay(mydisplay);
}
