/*  A program to scroll a line of text horizontally.  This
 *  is do to view portions of the line which is too long to fit into the
 *  viewing window.  A slider is used to move the viewing window along the line
 *  of text to bring the required continuous section of text into view.
 *
 *  Coded by:  Ross Maloney
 *  Date:      December 2008
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <string.h>
#include <stdio.h>

#define big_width 40
#define big_height 50
#define big_x_hot 0
#define big_y_hot 4
static unsigned char big_bits[] = {
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
   0x00, 0x00, 0xe0, 0xff, 0x00, 0x00, 0x00, 0xe0, 0xff, 0x00, 0x00, 0x00,
   0xe0, 0xff, 0x00, 0x00, 0x00, 0xe4, 0x00, 0xc0, 0xff, 0x00, 0xe6, 0x00,
   0xe0, 0x9d, 0x00, 0xe7, 0x00, 0xe0, 0xb7, 0xf0, 0xe1, 0x00, 0x7e, 0xe3,
   0xc0, 0xff, 0xe0, 0x7b, 0xe7, 0x80, 0xc0, 0xf7, 0xf8, 0xcd, 0x80, 0xc0,
   0x87, 0x60, 0xff, 0x80, 0xc1, 0x47, 0xe0, 0xff, 0x00, 0xff, 0xc0, 0xe1,
   0xff, 0x08, 0xd8, 0x00, 0x03, 0x00, 0x1c, 0xcc, 0x00, 0x06, 0x00, 0x32,
   0xe6, 0x00, 0x02, 0x00, 0x23, 0x9a, 0x80, 0x01, 0x08, 0xc1, 0x89, 0xff,
   0x00, 0x3f, 0xfe, 0xe8, 0x7f, 0x80, 0x7f, 0x60, 0xfc, 0x3f, 0xc0, 0xff,
   0xc0, 0xe6, 0x7b, 0xc0, 0xff, 0x81, 0xec, 0x73, 0x80, 0xff, 0xc3, 0x78,
   0xff, 0xc0, 0xff, 0x61, 0xf0, 0xbf, 0x00, 0x7f, 0x30, 0xe0, 0x80, 0x01,
   0x1f, 0x10, 0xe0, 0x80, 0x03, 0x0e, 0x10, 0xf8, 0x78, 0x00, 0x13, 0xe0,
   0xc7, 0x0c, 0x60, 0x11, 0x20, 0x40, 0x04, 0xd0, 0x10, 0x00, 0x40, 0x06,
   0x08, 0xe0, 0x03, 0x40, 0x02, 0x24, 0xe0, 0x02, 0x40, 0x03, 0x36, 0x60,
   0x42, 0x20, 0x03, 0x33, 0x20, 0x62, 0x22, 0x02, 0x3b, 0x00, 0xe4, 0x27,
   0x06, 0x26, 0x00, 0xe4, 0x34, 0x0c, 0x20, 0x00, 0xb6, 0x34, 0x18, 0x2c,
   0x00, 0x93, 0x15, 0x10, 0x2a, 0x00, 0x19, 0x0f, 0x08, 0x6b, 0x80, 0x04,
   0x07, 0x7f, 0x73, 0x80, 0x06, 0x03, 0x46, 0x7c, 0x40, 0x01, 0x01, 0x46,
   0x70, 0xc0, 0x00, 0x00, 0x66, 0x80, 0x01, 0x00, 0xfe, 0x26, 0x00, 0x02,
   0x00, 0xfe, 0x27, 0x00, 0xfe, 0xff, 0xff, 0x67, 0x00, 0x00, 0x00, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0x7f, 0x00, 0x00, 0xfe};

//#define hslider_width 14
//#define hslider_height 9
//#define hslider_x_hot 0
//#define hslider_y_hot 4
//static unsigned char hslider_bits[] = {
//   0x10, 0x02, 0x18, 0x06, 0x1c, 0x0e, 0xfe, 0x1f, 0xff, 0x3f, 0xfe, 0x1f,
//   0x1c, 0x0e, 0x18, 0x06, 0x10, 0x02};

int main(int argc, char *argv)
{
  Display        *mydisplay;
  Window         baseWindow, textWindow, sliderWindow;
  XSetWindowAttributes  myat;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         baseEvent;
  GC             mygc;
  XGCValues      myGCvalues;
  XFontStruct    *font1;
  XColor         white, black, grey;
  Cursor         blob;
  Pixmap         sliderGlyph;
  char *window_name = "Hscroll";
  char *icon_name   = "Hs";
  char *textline = "Six white boomers, Snow white boomers, Racing Santa Clause through the blazing sun.  Six white boomers, Snow white boomers, On his Australian run";
  int            screen_num, done, where;
  unsigned long  mymask;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

	       /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  black.pixel = BlackPixel(mydisplay, screen_num);
  white.pixel = WhitePixel(mydisplay, screen_num);
  grey.pixel = 0xd3d3d3;
  myat.border_pixel = black.pixel;
  myat.background_pixel = white.pixel;
  myat.event_mask = ExposureMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  baseWindow = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                             100, 100, 200, 200, 2,
			     DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);

	       /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseWindow, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseWindow, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseWindow, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, baseWindow, &iconName);

  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseWindow, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseWindow, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseWindow, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);

	       /* 4.  establish window resources */
  myGCvalues.background = white.pixel;
  myGCvalues.foreground = black.pixel;
  mymask = GCForeground | GCBackground;
  mygc = XCreateGC(mydisplay, baseWindow, mymask, &myGCvalues);
  sliderGlyph = XCreatePixmapFromBitmapData(mydisplay, baseWindow,
                         big_bits, big_width, big_height, 1, 0, 1);
  blob = XCreatePixmapCursor(mydisplay, sliderGlyph, sliderGlyph,
			 &white, &black,
                         big_x_hot, big_y_hot);
  font1 = XLoadQueryFont(mydisplay,
                    "-adobe-times-bold-r-normal--0-0-0-0-p-0-iso8859-1");

	       /* 5.  create all the other windows needed */
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  myat.background_pixel = black.pixel;
  textWindow = XCreateWindow(mydisplay, baseWindow, 30, 40, 140, 26, 2,
                             DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);
  myat.background_pixel = grey.pixel;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  sliderWindow = XCreateWindow(mydisplay, baseWindow, 30, 80, 140, 11, 2,
                             DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);
  XDefineCursor(mydisplay, sliderWindow, blob);

	       /* 6.  select events for each window */
	       /* 7.  map the windows */
  XMapWindow(mydisplay, baseWindow);
  XMapWindow(mydisplay, textWindow);
  XMapWindow(mydisplay, sliderWindow);

	       /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &baseEvent);
    switch ( baseEvent.type )  {
    case  Expose:
      XDrawImageString(mydisplay, textWindow, mygc,
                       0, 20, textline, strlen(textline));
      break;
    case  ButtonPress:
      break;
    }
  }

	       /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, baseWindow);
}
