/*  A program which produces a window containing a vertical slider bar.  The
 *  slider is picked up by clicking the left-hand mouse button over the slider.
 *  While that buttn is depressed the slider can be moved along the slider bed
 *  with the emd of the motion indicated by releasing that mouse button.  The
 *  coordinates of the slider are printed on the terminal screen as the slider
 *  is moved.
 *
 *  Coded by:  Ross Maloney
 *  Date:      February 2009
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <string.h>
#include <stdio.h>

#define arrow_width 16
#define arrow_height 16
static unsigned char arrow_bits[] = {
   0x00, 0x00, 0x06, 0x00, 0x1e, 0x00, 0x7c, 0x00, 0xfc, 0x01, 0xfc, 0x07,
   0xf8, 0x1f, 0xf8, 0x7f, 0xf8, 0x7f, 0xf0, 0x03, 0xf0, 0x07, 0xe0, 0x06,
   0xe0, 0x0c, 0xc0, 0x18, 0x80, 0x30, 0x00, 0x00};

#define mask_width 16
#define mask_height 16
static unsigned char mask_bits[] = {
   0x07, 0x00, 0x1f, 0x00, 0x7f, 0x00, 0xfe, 0x01, 0xfe, 0x07, 0xfe, 0x1f,
   0xfc, 0x3f, 0xfc, 0xff, 0xf8, 0xff, 0xf8, 0xff, 0xf8, 0x0f, 0xf0, 0x1f,
   0xf0, 0x3f, 0xe0, 0x7d, 0xe0, 0x79, 0xc0, 0x71};


int main(int argc, char *argv)
{
  Display        *mydisplay;
  Window         baseWindow, sliderWindow, sliderbedWindow;
  XSetWindowAttributes  myat;
  XSizeHints     wmsize;
  XWMHints       wmhints;
  XTextProperty  windowName, iconName;
  XEvent         baseEvent;
  GC             mygc;
  XGCValues      myGCvalues;
  XFontStruct    *font1;
  GC             Pgc;
  XGCValues      PGCValues;
  Pixmap         Ppattern, mask;
  unsigned long  Pmask;
  int            x, y;
  char *window_name = "Slider";
  char *icon_name   = "Sb";
  int            screen_num, done;
  unsigned long  mymask;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

	       /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  myat.border_pixel = BlackPixel(mydisplay, screen_num);
  myat.background_pixel = WhitePixel(mydisplay, screen_num);
  myat.event_mask = ExposureMask | ButtonPressMask | Button1MotionMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  baseWindow = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                             100, 100, 200, 200, 2,
			     DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);

	       /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseWindow, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseWindow, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseWindow, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, baseWindow, &iconName);

  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseWindow, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseWindow, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseWindow, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);

	       /* 4.  establish window resources */

	       /* 5.  create all the other windows needed */
  myat.background_pixel = 0xd3d3d3;
  myat.event_mask = ExposureMask | Button1MotionMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  sliderbedWindow = XCreateWindow(mydisplay, baseWindow, 90, 30, 11, 140, 2,
                             DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);
  myat.background_pixel = BlackPixel(mydisplay, screen_num);
  myat.event_mask = ExposureMask;
  mymask = CWBackPixel | CWBorderPixel | CWEventMask;
  sliderWindow = XCreateWindow(mydisplay, sliderbedWindow, 1, 0, 7, 14, 1,
                             DefaultDepth(mydisplay, screen_num), InputOutput,
			     DefaultVisual(mydisplay, screen_num),
			     mymask, &myat);
  Ppattern = XCreatePixmapFromBitmapData(mydisplay, baseWindow,
                               arrow_bits, arrow_width, arrow_height,
			       WhitePixel(mydisplay, screen_num),
			       BlackPixel(mydisplay, screen_num),
			       DefaultDepth(mydisplay, screen_num));
  mask = XCreatePixmapFromBitmapData(mydisplay, baseWindow,
                               mask_bits, mask_width, mask_height, 1, 0, 1);
  Pmask = GCForeground | GCBackground | GCClipMask;
  PGCValues.background = WhitePixel(mydisplay, screen_num);
  PGCValues.foreground = BlackPixel(mydisplay, screen_num);
  PGCValues.clip_mask = mask;
  Pgc = XCreateGC(mydisplay, baseWindow, Pmask, &PGCValues);

	       /* 6.  select events for each window */
	       /* 7.  map the windows */
  XMapWindow(mydisplay, baseWindow);
  XMapWindow(mydisplay, sliderbedWindow);
  XMapWindow(mydisplay, sliderWindow);

	       /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &baseEvent);
    switch ( baseEvent.type )  {
    case  Expose:
      break;
    case  ButtonPress:
      if ( baseEvent.xbutton.button == Button3 )  {
        x = baseEvent.xbutton.x;
        y = baseEvent.xbutton.y;
	printf("right click heard x = %d  y = %d\n", x, y);
	XSetClipOrigin(mydisplay, Pgc, 100, 80);
        XCopyPlane(mydisplay, Ppattern, baseWindow, Pgc, 0, 0,
                   arrow_width, arrow_height, 102, 80, 1);
}
      break;
    case  ButtonRelease:
      break;
    case  MotionNotify:
      printf("Moving to: x = %d   y = %d\n",
             baseEvent.xmotion.x, baseEvent.xmotion.y);
      XMoveWindow(mydisplay, sliderWindow, 1, baseEvent.xmotion.y - 7);
      break;
    }
  }

	       /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, baseWindow);
  XUnmapWindow(mydisplay, sliderbedWindow);
  XUnmapWindow(mydisplay, sliderWindow);
}
