/*  Thos program consists of a main window on which is placed a selection
 *  button.  The selection button is green in colour with the label 'selection'
 *  in pick characters.  By clicking the left mouse button on this selection
 *  button an option menu of 'flowers', 'pets', and 'quit' appears, each option
 *  labelled in blue with a pink background.  On moving the mouse pointer to
 *  each option, the pink background of the option changes to red.  Clicking the
 *  right mouse button on the 'quit' option terminates the program.
 *
 *  Coded by:  Ross Maloney
 *  Date:      July 2008
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>

int main(int argc, char *argv)
{
  Display       *mydisplay;
  XSetWindowAttributes  baseat, menuat, optionsat;
  Window        baseW, menuW, optionsW;
  XSizeHints    wmsize;
  XWMHints      wmhints;
  XTextProperty windowName, iconName;
  XEvent        myevent;
  XColor        exact, closest;
  GC            baseGC;
  XGCValues     myGCValues;
  char *window_name = "Pixmenus";
  char *icon_name   = "Pm";
  int           screen_num, done;
  unsigned long valuemask, blue, pink, red, green;

               /* 1.  open connection to the server */
  mydisplay = XOpenDisplay("");

	       /* 2.  create a top-level window */
  screen_num = DefaultScreen(mydisplay);
  baseat.background_pixel = WhitePixel(mydisplay, screen_num);
  baseat.border_pixel = BlackPixel(mydisplay, screen_num);
  valuemask = CWBackPixel | CWBorderPixel;
  baseW = XCreateWindow(mydisplay, RootWindow(mydisplay, screen_num),
                        300, 300, 350, 400, 3,
			DefaultDepth(mydisplay, screen_num), InputOutput,
			DefaultVisual(mydisplay, screen_num),
			valuemask, &baseat);

	       /* 3.  give the Window Manager hints */
  wmsize.flags = USPosition | USSize;
  XSetWMNormalHints(mydisplay, baseW, &wmsize);
  wmhints.initial_state = NormalState;
  wmhints.flags = StateHint;
  XSetWMHints(mydisplay, baseW, &wmhints);
  XStringListToTextProperty(&window_name, 1, &windowName);
  XSetWMName(mydisplay, baseW, &windowName);
  XStringListToTextProperty(&icon_name, 1, &iconName);
  XSetWMIconName(mydisplay, baseW, &iconName);

	       /* 4.  establish window resources */
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "red", &exact, &closest);
  red = closest.pixel;
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "blue", &exact, &closest);
  blue = closest.pixel;
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "pink", &exact, &closest);
  pink = closest.pixel;
  XAllocNamedColor(mydisplay, XDefaultColormap(mydisplay, screen_num),
                   "green", &exact, &closest);
  green = closest.pixel;

	       /* 5.  create all the other windows needed */
  valuemask = CWBackPixel | CWBorderPixel | CWEventMask;
  menuat.background_pixel = green;
  menuat.border_pixel = BlackPixel(mydisplay, screen_num);
  menuat.event_mask = ButtonPressMask | ButtonMotionMask |
                      EnterWindowMask | LeaveWindowMask;
  valuemask = CWBackPixel | CWBorderPixel | CWEventMask;
  menuW = XCreateSimpleWindow(mydisplay, baseW, 10, 50, 60, 20, 1,
                              BlackPixel(mydisplay, screen_num), red);
  XChangeWindowAttributes(mydisplay, menuW, valuemask, &menuat);

	       /* 6.  select events for each window */

	       /* 7.  map the windows */
  XMapWindow(mydisplay, baseW);
  XMapWindow(mydisplay, menuW);

	       /* 8.  enter the event loop */
  done = 0;
  while ( done == 0 )  {
    XNextEvent(mydisplay, &myevent);
    switch (myevent.type)  {
    case  ButtonPress:
      printf("entered\n");
      optionsW = XCreateSimpleWindow(mydisplay, baseW, 40, 60, 60, 80, 1,
                                     BlackPixel(mydisplay, screen_num), red);
      XMapWindow(mydisplay, optionsW);
      break;
    case  EnterNotify:
      XSetWindowBackground(mydisplay, menuW, blue);
      printf("entered\n");
      XClearWindow(mydisplay, menuW);
      XMapWindow(mydisplay, menuW);
      break;
    case  LeaveNotify:
      XSetWindowBackground(mydisplay, menuW, red);
      XClearWindow(mydisplay, menuW);
      XMapWindow(mydisplay, menuW);
      break;
    }
  }

	       /* 9.  clean up before exiting */
  XUnmapWindow(mydisplay, baseW);
  XDestroyWindow(mydisplay, baseW);
  XCloseDisplay(mydisplay);
}
